/*
  Antenna House PDF Tool API V5.0
  Java Interface sample program

  概要：利用の制限
  　　　内容のコピー関する制限を設定します。
  　　　暗号化済みのファイルの場合はその他のセキュリティ情報は保持されます。
  　　　40 bit RC4 暗号化の場合はコピーの可否とアクセシビリティ有効の可否が連動します。

  Copyright (C) 2018- Antenna House, Inc. All rights reserved.
  Antenna House is a trademark of [Antenna House, Inc.]

  Licensed under the Apache License, Version 2.0 (the "License");
  you may not use this file except in compliance with the License.
  You may obtain a copy of the License at

  http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.
*/

package cookbook;

import java.io.*;
import jp.co.antenna.ptl.*;

public class EncryptSetCopy {

    // そのクラスのusageを表示する関数
    public static void printUsage(){
        System.out.print("usage: java EncryptSetCopy in-pdf-file out-pdf-file ");
        System.out.println("コピー可否 out-pdf-ownerpass [in-pdf-password]");
        System.out.println("コピー可否\n0 : コピーを許可しない 1 : コピーを許可する");
        System.out.print("暗号化種類が40 bit RC4 の場合はコピーの可否が");
        System.out.println("アクセシビリティ有効の可否と連動します。");
    }

    /**
     * @param args the command line arguments
     */
    public static void main(String[] args) {
        if (args.length < 4)
        {
            printUsage(); // usageメッセージの表示
            return;
        }

        // コマンドライン引数の取得
        boolean permCopy = false;
        switch (args[2]) {
        case "0":
            permCopy = false;
            System.out.println("この文書の内容コピーを不許可に設定します。");
            break;
        case "1":
            permCopy = true;
            System.out.println("この文書の内容コピーを許可に設定します。");
            break;
        default:
            System.out.println("コピー可否は、0か1の数値で指定してください。");
            printUsage(); // usageメッセージの表示
            return;
        }
        String outOwnerPass = args[3];

        try (PtlParamInput inputFile = new PtlParamInput(args[0]);
             PtlParamOutput outputFile = new PtlParamOutput(args[1]);
             PtlPDFDocument doc = new PtlPDFDocument();
             BufferedReader br = new BufferedReader(new InputStreamReader(System.in)))
        {
            if (args.length > 4)
            {
                String password = (String)args[4];
                // パスワードのセット
                doc.setPassword(password);
            }

            // PDFファイルをロードします。
            doc.load(inputFile);

            // 暗号化の有無判定
            if (doc.isEncrypted())
            {
                // 暗号化情報の取得
                setPermCopy(doc, outOwnerPass, permCopy);
            }
            else
            {
                System.out.println("暗号化されていないファイルです。\n");
                System.out.println("コピーの可否を設定して暗号化します。");
                System.out.println("暗号化種類の設定をします。0から3の数値を指定してください。");
                System.out.println("それ以外の数値が設定された場合は 3 が選択されたものとして扱います。");
                System.out.println("暗号化種類\n0 : 40 bit RC4  1 : 128 bit RC4  2 : 128 bit AES  3 : 256 bit AES");
                int kind;
                try
                {
                    kind = Integer.parseInt(br.readLine());
                    // セキュリティ設定
                    switch (kind)
                    {
                    case 0:
                        // 40 bit RC4
                        encrypt40RC4setCopy(doc, outOwnerPass, permCopy);
                        break;
                    case 1:
                        // 128 bit RC4
                        encrypt128RC4setCopy(doc, outOwnerPass, permCopy);
                        break;
                    case 2:
                        // 128 bit AES
                        encrypt128AESsetCopy(doc, outOwnerPass, permCopy);
                        break;
                    case 3:
                    default:
                        // 256 bit AES
                        encrypt256AESsetCopy(doc, outOwnerPass, permCopy);
                        break;
                    }
                }
                catch (NumberFormatException nfe)
                {
                    System.out.println("暗号化の種類指定は整数を用いてください。");
                    nfe.printStackTrace();
                }
                
            }
            

            // ファイルに保存します。
            doc.save(outputFile);
        }
        catch (PtlException pex) {
            System.out.println("PtlException : ErrorCode = " + pex.getErrorCode() +
                               "\n  " + pex.getErrorMessage());
        }
        catch (Exception | Error ex) {
            System.out.println(ex.getMessage());
            ex.printStackTrace();
        }
        finally {
            System.out.println("-- 完了 --");
        }
    }



    public static void encrypt40RC4setCopy(PtlPDFDocument doc, String outOwnerPass, boolean permCopy)
        throws PtlException, IOException, Exception, Error
    {
        // 40 bit RC4
        System.out.println("40 bit RC4暗号化が選択されました。");
        try (PtlEncryptStandard40RC4 enc40 = new PtlEncryptStandard40RC4())
        {
            setCopyWithDefaultPermission(doc, outOwnerPass, permCopy, enc40);
        }

    }

    public static void encrypt128RC4setCopy(PtlPDFDocument doc, String outOwnerPass, boolean permCopy)
        throws PtlException, IOException, Exception, Error
    {
        // 128 bit RC4
        System.out.println("128 bit RC4暗号化が選択されました。");
        try (PtlEncryptStandard128RC4 enc128r = new PtlEncryptStandard128RC4())
        {
            setCopyWithDefaultPermission(doc, outOwnerPass, permCopy, enc128r);
        }
    }

    public static void encrypt128AESsetCopy(PtlPDFDocument doc, String outOwnerPass, boolean permCopy)
        throws PtlException, IOException, Exception, Error
    {
        // 128 bit AES
        System.out.println("128 bit AES暗号化が選択されました。");
        try (PtlEncryptStandard128AES enc128a = new PtlEncryptStandard128AES())
        {
            setCopyWithDefaultPermission(doc, outOwnerPass, permCopy, enc128a);
        }
    }

    public static void encrypt256AESsetCopy(PtlPDFDocument doc, String outOwnerPass, boolean permCopy)
        throws PtlException, IOException, Exception, Error
    {
        // 256 bit AES
        System.out.println("256 bit AES暗号化が選択されました。");
        try (PtlEncryptStandard256AES enc256 = new PtlEncryptStandard256AES())
        {
            setCopyWithDefaultPermission(doc, outOwnerPass, permCopy, enc256);
        }            
    }
    public static void setCopyWithDefaultPermission(PtlPDFDocument doc, String outOwnerPass,
                                                    boolean permCopy, PtlEncryptStandard enc)
        throws PtlException, IOException, Exception, Error
    {
        // 暗号化するコンテンツ
        // 文書の全てのコンテンツを暗号化します。
        enc.setEncryptComponent(PtlEncrypt.ENCRYPT_COMPONENT.ENCRYPT_ALL);
        // オーナーパスワード値の設定
        enc.setOwnerPassword(outOwnerPass);
        // encが40 bit RC4の場合はPtlEncryptPermissionType1を用いる必要がある
        if((enc.getMethod() == PtlEncryptStandard.STANDARD_METHOD_TYPE.METHOD_RC4) &&
           (enc.getKeyLength() == 40))
        {
            try (PtlEncryptPermissionType1 perms1 = new PtlEncryptPermissionType1())
            {
                // 印刷権限の設定
                // true : 印刷を許可する
                perms1.setPrint(true);

                // 変更権限の設定
                // ページ抽出を除く全ての動作を許可する。
                perms1.setModify(PtlEncryptPermissionType1.PERMISSION_MODIFY.PERM_MODIFY_MODYFYDOC);

                // テキスト、画像、その他の内容のコピーおよびアクセシビリティを有効にするかどうかの設定
                perms1.setCopy(permCopy);
                System.out.println("コピー許可の可否とアクセシビリティ有効の可否が連動して設定されます。");

                // ユーザアクセス許可フラグの設定
                enc.setPermission(perms1);
            }
        }
        // encが40 bit RC4以外場合はPtlEncryptPermissionType2を用いる必要がある
        else
        {
            try (PtlEncryptPermissionType2 perms2 = new PtlEncryptPermissionType2())
            {
                // 印刷権限の設定
                // PERM_PRINT_HIGH : 高解像度での印刷を許可
                perms2.setPrint(PtlEncryptPermissionType2.PERMISSION_PRINT.PERM_PRINT_HIGH);

                // 変更権限の設定
                // ページ抽出を除く全ての動作を許可
                perms2.setModify(PtlEncryptPermissionType2.PERMISSION_MODIFY.PERM_MODIFY_MODYFYDOC);

                // テキスト、画像、その他の内容のコピーを有効にするかどうかの設定
                perms2.setCopy(permCopy);

                // スクリーンリーダーデバイスのテキストアクセスを有効にするかどうかの設定
                // true: 有効にする
                perms2.setAccessibility(true);

                // ユーザアクセス許可フラグの設定
                enc.setPermission(perms2);
            }
        }
        // 暗号化情報の設定
        doc.setEncrypt(enc);
    }

    public static void setPermCopy(PtlPDFDocument doc, String outOwnerPass, boolean permCopy)
        throws PtlException, IOException, Exception, Error
    {
        try (PtlEncrypt encrypt = doc.getEncrypt())
        {
            // 標準セキュリティハンドラのメソッド取得
            if (encrypt.getFilterType() == PtlEncrypt.FILTER_TYPE.FILTER_STANDARD)
            {
                // getFilterType()に判定済みなので PtlEncryptStandard にダウンキャストする。
                PtlEncryptStandard encryptStandard = (PtlEncryptStandard)encrypt;
                encryptStandard.setOwnerPassword(outOwnerPass);
                // ユーザアクセス許可フラグの取得
                try (PtlEncryptPermission perms = encryptStandard.getPermission())
                {
                    // タイプ取得
                    switch (perms.getType())
                    {
                    case PERM_TYPE1:
                        // getType()で判定されたのでpermsをダウンキャストできる。
                        PtlEncryptPermissionType1 perms1 = (PtlEncryptPermissionType1)perms;
                        // テキスト、画像、その他の内容のコピーおよびアクセシビリティを有効にするかどうかの設定
                        perms1.setCopy(permCopy);
                        System.out.println("40 bit RC4 暗号化をされたファイルです。");
                        System.out.print("コピー許可の可否とアクセシビリティ有効の可否が");
                        System.out.println("連動して設定されます。");
                        encryptStandard.setPermission(perms1);
                        break;
                        
                    case PERM_TYPE2:
                        // getType()で判定されたのでpermsをダウンキャストできる。
                        PtlEncryptPermissionType2 perms2 = (PtlEncryptPermissionType2)perms;
                        // テキスト、画像、その他の内容のコピーを有効にするかどうかの設定
                        perms2.setCopy(permCopy);
                        encryptStandard.setPermission(perms2);
                        break;
                    }
                }
                // 暗号化情報の設定
                doc.setEncrypt(encryptStandard);
            }
            else
            {
                System.out.println("標準セキュリティハンドラ以外のセキュリティハンドラが使われたPDFです。");
                System.out.println("このPDFの権限を変更することはできません。");
                return;
            }
        }
    }

}

