/*
  Antenna House PDF Tool API V5.0
  Java Interface sample program

  概要：印刷不可セキュリティの設定
  　　　印刷の可否について設定します。
  　　　暗号化済みのファイルの場合はその他のセキュリティ情報は保持されます。

  Copyright (C) 2018- Antenna House, Inc. All rights reserved.
  Antenna House is a trademark of [Antenna House, Inc.]

  Licensed under the Apache License, Version 2.0 (the "License");
  you may not use this file except in compliance with the License.
  You may obtain a copy of the License at

  http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.
*/

package cookbook;

import java.io.*;
import jp.co.antenna.ptl.*;

public class EncryptSetPrint {

    // そのクラスのusageを表示する関数
    public static void printUsage(){
        System.out.print("usage: java EncryptSetPrint in-pdf-file out-pdf-file ");
        System.out.println("印刷権限種類 out-pdf-ownerpass [in-pdf-password]");
        System.out.println("印刷権限種類\n0 : 印刷不可  1 : 低解像度で可  2 : 高解像度で可");
        System.out.print("暗号化種類が40 bit RC4 の場合は解像度の指定はできず、");
        System.out.println("印刷の可否についての情報だけが保持・指定されます。");
    }

    /**
     * @param args the command line arguments
     */
    public static void main(String[] args) {
        if (args.length < 4)
        {
            printUsage(); // usageメッセージの表示
            return;
        }

        // コマンドライン引数の取得
        int permPrint = -1;
        try
        {
            permPrint = Integer.parseInt(args[2]);
            switch (permPrint) {
            case 0:
            case 1:
            case 2:
                break;
            default:
                System.out.println("印刷権限種類には、0から2の数値を指定してください。");
                printUsage(); // usageメッセージの表示
                return;
            }
        }
        catch(NumberFormatException nfe)
        {
            System.out.println("印刷権限種類には、0から2の数値を指定してください。");
            printUsage(); // usageメッセージの表示
        }
        String outOwnerPass = args[3];

        
        try (PtlParamInput inputFile = new PtlParamInput(args[0]);
             PtlParamOutput outputFile = new PtlParamOutput(args[1]);
             PtlPDFDocument doc = new PtlPDFDocument();
             BufferedReader br = new BufferedReader(new InputStreamReader(System.in)))
        {
            if (args.length > 4)
            {
                String password = (String)args[4];
                // パスワードのセット
                doc.setPassword(password);
            }

            // PDFファイルをロードします。
            doc.load(inputFile);

            // 暗号化の有無判定
            if (doc.isEncrypted())
            {
                // 暗号化情報の取得
                setPermPrint(doc, outOwnerPass, permPrint);
            }
            else
            {
                System.out.println("暗号化されていないファイルです。\n");
                System.out.println("印刷権限を設定して暗号化します。");
                System.out.println("暗号化種類の設定をします。0から3の数値を指定してください。");
                System.out.println("それ以外の数値が設定された場合は 3 が選択されたものとして扱います。");
                System.out.println("暗号化種類");
                System.out.println("0 : 40 bit RC4  1 : 128 bit RC4  2 : 128 bit AES  3 : 256 bit AES");
                int kind = Integer.parseInt(br.readLine());
                // セキュリティ設定
                switch (kind)
                {
                case 0:
                    // 40 bit RC4
                    encrypt40RC4setPrint(doc, outOwnerPass, permPrint);
                    break;
                case 1:
                    // 128 bit RC4
                    encrypt128RC4setPrint(doc, outOwnerPass, permPrint);
                    break;
                case 2:
                    // 128 bit AES
                    encrypt128AESsetPrint(doc, outOwnerPass, permPrint);
                    break;
                case 3:
                default:
                    // 256 bit AES
                    encrypt256AESsetPrint(doc, outOwnerPass, permPrint);
                    break;
                }
                
            }
            
            // ファイルに保存します。
            doc.save(outputFile);
        }
        catch (PtlException pex) {
            System.out.println("PtlException : ErrorCode = " + pex.getErrorCode() +
                               "\n  " + pex.getErrorMessage());
        }
        catch (Exception | Error ex) {
            System.out.println(ex.getMessage());
            ex.printStackTrace();
        }
        finally {
            System.out.println("-- 完了 --");
        }
    }



    public static void encrypt40RC4setPrint(PtlPDFDocument doc, String outOwnerPass, int permPrint)
        throws PtlException, Exception, Error
    {
        // 40 bit RC4
        System.out.println("40 bit RC4暗号化が選択されました。");
        try (PtlEncryptStandard40RC4 enc40 = new PtlEncryptStandard40RC4())
        {
            setPrintWithDefaultPermission(doc, outOwnerPass, permPrint, enc40);
        }
    }

    public static void encrypt128RC4setPrint(PtlPDFDocument doc, String outOwnerPass, int permPrint)
        throws PtlException, Exception, Error
    {
        // 128 bit RC4
        System.out.println("128 bit RC4暗号化が選択されました。");
        try (PtlEncryptStandard128RC4 enc128r = new PtlEncryptStandard128RC4())
        {
            setPrintWithDefaultPermission(doc, outOwnerPass, permPrint, enc128r);
        }
    }

    public static void encrypt128AESsetPrint(PtlPDFDocument doc, String outOwnerPass, int permPrint)
        throws PtlException, Exception, Error
    {
        // 128 bit AES
        System.out.println("128 bit AES暗号化が選択されました。");
        try (PtlEncryptStandard128AES enc128a = new PtlEncryptStandard128AES())
        {
            setPrintWithDefaultPermission(doc, outOwnerPass, permPrint, enc128a);
        }
    }

    public static void encrypt256AESsetPrint(PtlPDFDocument doc, String outOwnerPass, int permPrint)
        throws PtlException, Exception, Error
    {
        // 256 bit AES
        System.out.println("256 bit AES暗号化が選択されました。");
        try (PtlEncryptStandard256AES enc256 = new PtlEncryptStandard256AES())
        {
            setPrintWithDefaultPermission(doc, outOwnerPass, permPrint, enc256);
        }
    }

    public static void setPrintWithDefaultPermission(PtlPDFDocument doc, String outOwnerPass,
                                                     int permPrint, PtlEncryptStandard enc)
        throws PtlException, Exception, Error
    {

        // 暗号化するコンテンツ
        // 文書の全てのコンテンツを暗号化します。
        enc.setEncryptComponent(PtlEncrypt.ENCRYPT_COMPONENT.ENCRYPT_ALL);
        // オーナーパスワード値の設定
        enc.setOwnerPassword(outOwnerPass);
        // encが40 bit RC4の場合はPtlEncryptPermissionType1を用いる必要がある
        if((enc.getMethod() == PtlEncryptStandard.STANDARD_METHOD_TYPE.METHOD_RC4) &&
           (enc.getKeyLength() == 40))
        {
            try (PtlEncryptPermissionType1 perms1 = new PtlEncryptPermissionType1())
            {
                switch(permPrint)
                {
                case 0:
                    perms1.setPrint(false);
                    break;
                case 1:
                case 2:
                    perms1.setPrint(true);
                    break;
                }
                // 変更権限の設定
                // ページ抽出を除く全ての動作を許可する。
                perms1.setModify(PtlEncryptPermissionType1.PERMISSION_MODIFY.PERM_MODIFY_MODYFYDOC);

                // テキスト、画像、その他の内容のコピーおよびアクセシビリティを有効にするかどうかの設定
                // true : 有効にする
                perms1.setCopy(true);

                // ユーザアクセス許可フラグの設定
                enc.setPermission(perms1);
            }
        }
        // encが40 bit RC4以外場合はPtlEncryptPermissionType2を用いる必要がある
        else
        {
            try (PtlEncryptPermissionType2 perms2 = new PtlEncryptPermissionType2())
            {
                // 印刷権限の設定
                switch(permPrint)
                {
                case 0:
                default:
                    perms2.setPrint(PtlEncryptPermissionType2.PERMISSION_PRINT.PERM_PRINT_NOT_ALLOWED);
                    break;
                case 1:
                    perms2.setPrint(PtlEncryptPermissionType2.PERMISSION_PRINT.PERM_PRINT_LOW);
                    break;
                case 2:
                    perms2.setPrint(PtlEncryptPermissionType2.PERMISSION_PRINT.PERM_PRINT_HIGH);
                    break;
                }

                // 変更権限の設定
                // ページ抽出を除く全ての動作を許可
                perms2.setModify(PtlEncryptPermissionType2.PERMISSION_MODIFY.PERM_MODIFY_MODYFYDOC);

                // テキスト、画像、その他の内容のコピーを有効にするかどうかの設定
                // true: 有効にする
                perms2.setCopy(true);

                // スクリーンリーダーデバイスのテキストアクセスを有効にするかどうかの設定
                // true: 有効にする
                perms2.setAccessibility(true);

                // ユーザアクセス許可フラグの設定
                enc.setPermission(perms2);
            }
        }

        // 暗号化情報の設定
        doc.setEncrypt(enc);
        
    }
    
    public static void setPermPrint(PtlPDFDocument doc, String outOwnerPass, int permPrint)
        throws PtlException, Exception, Error
    {
        try (PtlEncrypt encrypt = doc.getEncrypt())
        {
            
            // 標準セキュリティハンドラのメソッド取得
            if (encrypt.getFilterType() == PtlEncrypt.FILTER_TYPE.FILTER_STANDARD)
            {
                // getFilterType()に判定済みなので PtlEncryptStandard にダウンキャストする。
                PtlEncryptStandard encryptStandard = (PtlEncryptStandard)encrypt;
                encryptStandard.setOwnerPassword(outOwnerPass);
                // ユーザアクセス許可フラグの取得
                try (PtlEncryptPermission perms = encryptStandard.getPermission())
                {
                    // タイプ取得
                    switch (perms.getType())
                    {
                    case PERM_TYPE1:
                        // getType()で判定されたのでpermsをダウンキャストできる。
                        PtlEncryptPermissionType1 perms1 = (PtlEncryptPermissionType1)perms;
                        // 印刷権限設定
                        switch(permPrint)
                        {
                        case 0:
                            perms1.setPrint(false);
                            break;
                        case 1:
                        case 2:
                            perms1.setPrint(true);
                            break;
                        }
                        encryptStandard.setPermission(perms1);
                        break;
                        
                    case PERM_TYPE2:
                        // getType()で判定されたのでpermsをダウンキャストできる。
                        PtlEncryptPermissionType2 perms2 = (PtlEncryptPermissionType2)perms;
                        switch(permPrint)
                        {
                        case 0:
                        default:
                            perms2.setPrint(PtlEncryptPermissionType2.PERMISSION_PRINT.PERM_PRINT_NOT_ALLOWED);
                            break;
                        case 1:
                            perms2.setPrint(PtlEncryptPermissionType2.PERMISSION_PRINT.PERM_PRINT_LOW);
                            break;
                        case 2:
                            perms2.setPrint(PtlEncryptPermissionType2.PERMISSION_PRINT.PERM_PRINT_HIGH);
                            break;
                        }
                        encryptStandard.setPermission(perms2);
                        break;
                    }
                }
                // 暗号化情報の設定
                doc.setEncrypt(encryptStandard);
            }
            else
            {
                System.out.println("標準セキュリティハンドラ以外のセキュリティハンドラが使われたPDFです。");
                System.out.println("このPDFの印刷権限を変更することはできません。");
                return;
            }
        }
    }
}

