 /*
  Antenna House PDF Tool API V5.0
  Java Interface sample program

  概要：セキュリティ情報の設定・オーナーパスワードのみ設定する。

  Copyright (C) 2018- Antenna House, Inc. All rights reserved.
  Antenna House is a trademark of [Antenna House, Inc.]

  Licensed under the Apache License, Version 2.0 (the "License");
  you may not use this file except in compliance with the License.
  You may obtain a copy of the License at

  http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.
*/

package cookbook;

import jp.co.antenna.ptl.*;

public class EncryptWithOwnerPass {

    // そのクラスのusageを表示する関数
    public static void printUsage(){
        System.out.print("usage: java EncryptWithOwnerPass in-pdf-file out-pdf-file");
        System.out.println("暗号化種類 out-pdf-ownerpass [in-pdf-password]\n");
        System.out.println("暗号化種類");
        System.out.println("0 : 40 bit RC4  1 : 128 bit RC4  2 : 128 bit AES  3 : 256 bit AES");
    }

    /**
     * @param args the command line arguments
     */
    public static void main(String[] args) {
        if (args.length < 4)
        {
            printUsage(); // usageメッセージの表示
            return;
        }

        int kind;
        try
        {
            kind =  Integer.parseInt(args[2]);
            switch (kind) {
            case 0:
            case 1:
            case 2:
            case 3:
                break;
            default:
                System.out.println("暗号化種類には、0から3の数値を指定してください。");
                printUsage(); // usageメッセージの表示
                return;
            }
        }
        catch(NumberFormatException nfe)
        {
            System.out.println("暗号化種類には、0から3の数値を指定してください。");
            printUsage(); // usageメッセージの表示
            return;
        }
        // コマンドライン引数の取得
        String outOwnerpass = args[3];
        try (PtlParamInput inputFile = new PtlParamInput(args[0]);
             PtlParamOutput outputFile = new PtlParamOutput(args[1]);
             PtlPDFDocument doc = new PtlPDFDocument())
        {
            if (args.length > 4)
            {
                String password = (String)args[4];
                // パスワードのセット
                doc.setPassword(password);
            }

            // PDFファイルをロードします。
            doc.load(inputFile);
            
            // セキュリティ設定
            switch (kind)
            {
            case 0:
                // 40 bit RC4
                encrypt40RC4withOwnerpass(doc, outOwnerpass);
                break;
            case 1:
                // 128 bit RC4
                encrypt128RC4withOwnerpass(doc, outOwnerpass);
                break;
            case 2:
                // 128 bit AES
                encrypt128AESwithOwnerpass(doc, outOwnerpass);
                break;
            case 3:
                // 256 bit AES
                encrypt256AESwithOwnerpass(doc, outOwnerpass);
                break;
            }

            // ファイルに保存します。
            doc.save(outputFile);
        }
        catch (PtlException pex) {
            System.out.println("PtlException : ErrorCode = " + pex.getErrorCode() +
                               "\n  " + pex.getErrorMessage());
        }
        catch (Exception | Error ex) {
            System.out.println(ex.getMessage());
            ex.printStackTrace();
        }
        finally {
            System.out.println("-- 完了 --");
        }
    }

    public static void encrypt40RC4withOwnerpass(PtlPDFDocument doc, String outOwnerpass)
        throws PtlException, Exception, Error
    {
        // 40 bit RC4
        System.out.println("40 bit RC4暗号化の設定をします。");
        try (PtlEncryptStandard40RC4 enc40 = new PtlEncryptStandard40RC4())
        {
            setEncryptWithOwnerpass(doc, outOwnerpass, enc40);
        }
    }

    public static void encrypt128RC4withOwnerpass(PtlPDFDocument doc, String outOwnerpass)
        throws PtlException, Exception, Error
    {
        // 128 bit RC4
        System.out.println("128 bit RC4暗号化の設定をします。");
        try (PtlEncryptStandard128RC4 enc128r = new PtlEncryptStandard128RC4())
        {
            setEncryptWithOwnerpass(doc, outOwnerpass, enc128r);
        }
    }

    public static void encrypt128AESwithOwnerpass(PtlPDFDocument doc, String outOwnerpass)
        throws PtlException, Exception, Error
    {
        // 128 bit AES
        System.out.println("128 bit AES暗号化の設定をします。");
        try (PtlEncryptStandard128AES enc128a = new PtlEncryptStandard128AES())
        {
            setEncryptWithOwnerpass(doc, outOwnerpass, enc128a);
        }
    }

    public static void encrypt256AESwithOwnerpass(PtlPDFDocument doc, String outOwnerpass)
        throws PtlException, Exception, Error
    {
        // 256 bit AES
        System.out.println("256 bit AES暗号化の設定をします。");
        try (PtlEncryptStandard256AES enc256 = new PtlEncryptStandard256AES())
        {
            setEncryptWithOwnerpass(doc, outOwnerpass, enc256);
        }
    }

    public static void setEncryptWithOwnerpass(PtlPDFDocument doc, String outOwnerpass, PtlEncryptStandard enc)
        throws PtlException, Exception, Error
    {
        // 暗号化するコンテンツを設定
        // ENCRYPT_ALL : 全て暗号化
        enc.setEncryptComponent(PtlEncrypt.ENCRYPT_COMPONENT.ENCRYPT_ALL);
        // ユーザパスワード値の設定
        enc.setOwnerPassword(outOwnerpass);
        
        // ユーザアクセス許可フラグの取得・設定
        // タイプ取得
        if((enc.getMethod() == PtlEncryptStandard.STANDARD_METHOD_TYPE.METHOD_RC4) &&
           (enc.getKeyLength() == 40))
        {
            // RC4 40 ビット 暗号化での権限設定
            try (PtlEncryptPermissionType1 perms1 = new PtlEncryptPermissionType1())
            {
                // 印刷権限の設定
                // false: 印刷を許可しない
                perms1.setPrint(false);

                // 変更権限の設定
                // PERM_MODIFY_NOT_ALLOWED : 許可しない
                perms1.setModify(PtlEncryptPermissionType1.PERMISSION_MODIFY.PERM_MODIFY_NOT_ALLOWED);

                // テキスト、画像、その他の内容のコピーおよびアクセシビリティを有効にするかどうかの設定
                // false: 有効にしない
                perms1.setCopy(false);

                // ユーザアクセス許可フラグの設定
                enc.setPermission(perms1);
            }
        }
        else
        {
            // RC4 40 ビット以外の暗号化での権限設定
            try (PtlEncryptPermissionType2 perms2 = new PtlEncryptPermissionType2())
            {
                // 印刷権限の設定
                // PERM_PRINT_NOT_ALLOWED : 許可しない
                perms2.setPrint(PtlEncryptPermissionType2.PERMISSION_PRINT.PERM_PRINT_NOT_ALLOWED);

                // 変更権限の設定
                // PERM_MODIFY_NOT_ALLOWED : 許可しない
                perms2.setModify(PtlEncryptPermissionType2.PERMISSION_MODIFY.PERM_MODIFY_NOT_ALLOWED);

                // テキスト、画像、その他の内容のコピーを有効にするかどうかの設定
                // false : 有効にしない
                perms2.setCopy(false);

                // スクリーンリーダーデバイスのテキストアクセスを有効にするかどうかの設定
                // false : 有効にしない
                perms2.setAccessibility(false);

                // ユーザアクセス許可フラグの設定
                enc.setPermission(perms2);
            }
        }
        doc.setEncrypt(enc);
    }
}

